﻿<System.ComponentModel.DefaultEvent("ValueChanged")> _
Public Class AdobeTrackBar
    Inherits Control

    Private Const THUMB_SIZE As Integer = 15
    Private Const TRACK_SIZE As Integer = 10
    Private Const SIDE_PADDING As Integer = 15

#Region "Events"
    Public Event ValueChanged As EventHandler

    Protected Overridable Sub OnValueChanged(ByVal e As EventArgs)
        RaiseEvent ValueChanged(Me, e)
    End Sub
#End Region

#Region "Constructor"
    Public Sub New()
        Me.DoubleBuffered = True
        Me.Height = 25
        Me.SetStyle(ControlStyles.AllPaintingInWmPaint Or ControlStyles.UserPaint Or _
                    ControlStyles.OptimizedDoubleBuffer Or ControlStyles.Selectable Or _
                    ControlStyles.SupportsTransparentBackColor Or _
                    ControlStyles.FixedHeight, True)
    End Sub
#End Region

#Region "Properties"
    Private m_Value As Integer = 0
    Private m_Maximum As Integer = 100

    ''' <summary>
    ''' The value to be displayed in the <see cref="AdobeTrackBar" />.
    ''' </summary>
    Public Property Value() As Integer
        Get
            Return m_Value
        End Get
        Set(ByVal value As Integer)
            If value < 0 OrElse value > Me.Maximum Then _
                Throw New ArgumentOutOfRangeException("value")
            m_Value = value
            Me.Invalidate()
            Me.OnValueChanged(EventArgs.Empty)
        End Set
    End Property

    ''' <summary>
    ''' The maximum value that can be displayed in the <see cref="AdobeTrackBar" />.
    ''' </summary>
    Public Property Maximum() As Integer
        Get
            Return m_Maximum
        End Get
        Set(ByVal value As Integer)
            If value <= 0 Then _
                Throw New ArgumentOutOfRangeException("value")
            m_Maximum = value
            If Me.Value > Me.Maximum Then
                Me.Value = Me.Maximum
            Else
                Me.Invalidate()
            End If
        End Set
    End Property

    Protected ReadOnly Property EffectiveWidth() As Integer
        Get
            Return Me.Width - SIDE_PADDING * 2
        End Get
    End Property

    <System.ComponentModel.Browsable(False)> _
    Public Overrides Property Text() As String
        Get
            Return MyBase.Text
        End Get
        Set(ByVal value As String)
            MyBase.Text = value
        End Set
    End Property
#End Region

#Region "Drawing"
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        Dim g As Graphics = e.Graphics
        g.SmoothingMode = Drawing2D.SmoothingMode.HighQuality

        'Draw the focus rectangle
        If (m_State And AdobeButtonState.Focus) > 0 Then
            Dim rect As Rectangle = Me.ClientRectangle
            rect.Width -= 1
            rect.Height -= 1
            Using p As New Pen(AdobeColors.ControlFocusColor)
                g.DrawRectangle(p, rect)
            End Using
        End If

        'Draw the track
        Dim r As New Rectangle(SIDE_PADDING, Me.Height \ 2 - TRACK_SIZE \ 2, Me.EffectiveWidth, TRACK_SIZE)
        Using b As New SolidBrush(AdobeColors.ControlGenericColor)
            g.FillRoundRectangle(b, r, TRACK_SIZE)
        End Using
        Using p As New Pen(AdobeColors.BorderColor)
            g.DrawRoundRectangle(p, r, TRACK_SIZE)
        End Using

        'Draw the thumb
        r = New Rectangle(CInt(Me.Value / Me.Maximum * Me.EffectiveWidth - THUMB_SIZE / 2 + SIDE_PADDING), Me.Height \ 2 - THUMB_SIZE \ 2, THUMB_SIZE, THUMB_SIZE)
        If (m_State And AdobeButtonState.Pushed) > 0 Then
            Using b As New SolidBrush(AdobeColors.ButtonPressedColor)
                g.FillEllipse(b, r)
            End Using
        ElseIf (m_State And AdobeButtonState.Hover) > 0 Then
            Using b As New Drawing2D.LinearGradientBrush(r, AdobeColors.ButtonHighlightGradientColor, AdobeColors.ControlHighlightColor, Drawing2D.LinearGradientMode.Vertical)
                g.FillEllipse(b, r)
            End Using
        Else
            Using b As New Drawing2D.LinearGradientBrush(r, AdobeColors.ButtonGradientColor, AdobeColors.ControlGenericColor, Drawing2D.LinearGradientMode.Vertical)
                g.FillEllipse(b, r)
            End Using
        End If

        'Draw the thumb's border
        Using p As New Pen(AdobeColors.BorderColor)
            g.DrawEllipse(p, r)
        End Using
    End Sub
#End Region

#Region "Mouse Events"
    Private m_State As AdobeButtonState = AdobeButtonState.Default

    Protected Overrides Sub OnMouseDown(ByVal e As System.Windows.Forms.MouseEventArgs)
        m_State = m_State Or AdobeButtonState.Pushed
        Dim v As Integer = CInt((e.X - SIDE_PADDING) / Me.EffectiveWidth * Me.Maximum)
        Me.Value = Math.Max(0, Math.Min(v, Me.Maximum))
        MyBase.OnMouseDown(e)
    End Sub

    Protected Overrides Sub OnMouseUp(ByVal e As System.Windows.Forms.MouseEventArgs)
        m_State = m_State And Not AdobeButtonState.Pushed
        Me.Invalidate()
        MyBase.OnMouseUp(e)
    End Sub

    Protected Overrides Sub OnMouseMove(ByVal e As System.Windows.Forms.MouseEventArgs)
        If (m_State And AdobeButtonState.Pushed) > 0 Then
            Dim v As Integer = CInt((e.X - SIDE_PADDING) / Me.EffectiveWidth * Me.Maximum)
            Me.Value = Math.Max(0, Math.Min(v, Me.Maximum))
        End If
        MyBase.OnMouseMove(e)
    End Sub

    Protected Overrides Sub OnMouseEnter(ByVal e As System.EventArgs)
        m_State = m_State Or AdobeButtonState.Hover
        Me.Invalidate()
        MyBase.OnMouseEnter(e)
    End Sub

    Protected Overrides Sub OnMouseLeave(ByVal e As System.EventArgs)
        m_State = m_State And Not AdobeButtonState.Hover
        Me.Invalidate()
        MyBase.OnMouseLeave(e)
    End Sub
#End Region

#Region "Focus Events"
    Protected Overrides Sub OnEnter(ByVal e As System.EventArgs)
        m_State = m_State Or AdobeButtonState.Focus
        Me.Invalidate()
        MyBase.OnEnter(e)
    End Sub

    Protected Overrides Sub OnLeave(ByVal e As System.EventArgs)
        m_State = m_State And Not AdobeButtonState.Focus
        Me.Invalidate()
        MyBase.OnLeave(e)
    End Sub
#End Region

#Region "Keyboard Events"
    Protected Overrides Sub OnKeyDown(ByVal e As System.Windows.Forms.KeyEventArgs)
        MyBase.OnKeyDown(e)
    End Sub
#End Region

End Class
